package express;

import express.http.HttpRequest;

/**
 * @author Simon Reinisch
 * Router interface for express
 */
public interface Router {

  /**
   * Add an middleware which will be called before each request-type listener will be fired.
   *
   * @param middleware An middleware which will be fired on every request-method and  path.
   */
  void use(HttpRequest middleware);

  /**
   * Add an middleware which will be called before each request-type listener will be fired.
   *
   * @param context    The context where the middleware should listen.
   * @param middleware An middleware which will be fired if the context matches the request-path.
   */
  void use(String context, HttpRequest middleware);

  /**
   * Add an middleware which will be called before each request-type listener will be fired.
   *
   * @param context       The context where the middleware should listen for the request handler..
   * @param requestMethod And type of request-method eg. GET, POST etc.
   * @param middleware    An middleware which will be fired if the context matches the request-method- and  path.
   */
  void use(String context, String requestMethod, HttpRequest middleware);

  /**
   * Add an listener for all request methods and contexts.
   *
   * @param request Will be fired on all requests.
   */
  void all(HttpRequest request);

  /**
   * Adds an handler for a specific context.
   *
   * @param context The context.
   * @param request An listener which will be fired if the context matches the request-path.
   */
  void all(String context, HttpRequest request);

  /**
   * Adds an handler for a specific context and method.
   * You can use a star '*' to match every context / request-method.
   *
   * @param context       The context.
   * @param requestMethod The request method.
   * @param request       An listener which will be fired if the context matches the request-path.
   */
  void all(String context, String requestMethod, HttpRequest request);

  /**
   * Add an listener for GET request's.
   *
   * @param context The context.
   * @param request An listener which will be fired if the context matches the request-path.
   */
  void get(String context, HttpRequest request);

  /**
   * Add an listener for POST request's.
   *
   * @param context The context.
   * @param request An listener which will be fired if the context matches the request-path.
   */
  void post(String context, HttpRequest request);

  /**
   * Add an listener for PUT request's.
   *
   * @param context The context for the request handler..
   * @param request An listener which will be fired if the context matches the request-path.
   */
  void put(String context, HttpRequest request);

  /**
   * Add an listener for DELETE request's.
   *
   * @param context The context.
   * @param request An listener which will be fired if the context matches the request-path.
   */
  void delete(String context, HttpRequest request);

  /**
   * Add an listener for PATCH request's.
   *
   * @param context The context.
   * @param request An listener which will be fired if the context matches the request-path.
   */
  void patch(String context, HttpRequest request);

}
